/* anatabacus.js: Dialog for EPI distortion correction plugin 'anatabacus' 
Content: This EPI distortion correction bundle consists of anatabacus.js, anatabacus.ui and anatabacus.dylib/dll/so. 
Installation: place all in /Documents/BVQXExtensions/Plugins_32/64/ folder.
Functionality: Undistort functional EPI data.
Usage: In the BrainVoyager menu, select 'Plugins' > 'anatabacus...'.

History: 
03-03-14: 
- in unit calculations, remove `partial Fourier' and replace with GRAPPA/SENSE `acceleration factor'; adapt calculation (from line 780)
- line 731: replace `rows in image' by 'matrix size y' 
19-12-13: added getFileType(filestage, fileid)  
18-12-13: added getVMRFile() function, getFileExtension(), getFileTypeFromExtension()
05-12-13: added VMR button and label
06-06-12: switched checkerrorstatus() off at end of procedure
05-10-11: closed and re-opened FMRs after Evaluation
03-10-11: added extendFileName() function
07-06-11: added chbPhaseToRadians, chbMaskLaplacian for unwrapping
28-09-10: removed echospacing widgets, debugging.
23-09-10: upgraded to BrainVoyager QX 2.2

Hester Breman, Brain Innovation, 2013
*/

var scriptObj 					= new Object; 
scriptObj.scriptNameForUser 	= "anatabacus"; 
scriptObj.dialogFileName 		= "anatabacus_v110.ui"; 
scriptObj.dialogAccessName      = "anatabacusDlg"; 
BrainVoyagerQX.PrintToLog("Anatabacus dialog started...");

// filetype (FMR or NIFTI)
const UNKNOWN_FILE_TYPE     = 0;
const BV_TYPE_VMR           = 20;
const BV_TYPE_FMR           = 21;
const NIFTI_FILE_TYPE       = 1; 
const NIFTI_PAIR_FILE_TYPE  = 2;

// stage of file(s) to be collected
const FILE_STAGE_NOT_SET                    = 0;
const FILE_STAGE_PHASE_UNWRAP               = 1; // unwrap
const FILE_STAGE_PHASE_COMPUTE_B0           = 2; // calculate field inhomogeneity
const FILE_STAGE_PHASE_COMPUTE_PIXSH        = 3; // calculate pixelshift
const FILE_STAGE_EPI_UNDISTORT              = 4; // undistort
const FILE_STAGE_EPI_REQUIRED               = 5; // any step requiring EPI
const FILE_STAGE_EPI_UNDISTORT_WITH_COREG   = 6;

// conversions
const CONVERT_OPTION_UNKNOWN = 0;
const CONVERT_SI_UNIT_MIN2SEC = 1;
const CONVERT_SI_UNIT_MSEC2SEC = 2;
const CONVERT_SI_UNIT_MICROSEC2SEC = 3;
const CONVERT_SI_UNIT_NANOSEC2SEC = 4;
const CONVERT_SI_UNIT_MM2M = 5;
const CONVERT_SI_UNIT_CM2M = 6;
const CONVERT_SI_UNIT_KHZ2HZ = 7;
const CONVERT_SI_UNIT_MHZ2HZ = 8;
const CONVERT_SI_UNIT_GHZ2HZ = 9;
const CONVERT_CART2POL = 10;
const CONVERT_POL2CART = 11;
const CONVERT_DEG2RAD = 12;
const CONVERT_RAD2DEG = 13;
const CONVERT_RAD2HZ = 14;
const CONVERT_PPM2HZ = 15;
const CONVERT_ESP2BWPE = 16;

// global variables
var cancel = false;
var nrofcycles = 0;
var nrofiterations = 0;
var do3DFMG = true;
var nrofiterationsrelax = 0;
var regularisation = 0.0;
var phase2radians = true;
var masklaplacian = false;
var bandwidthpe = 0;
var echotimediff = 0.0;
var phasencodingdir = 0;
var apply2pi = false;
var startfileid, epifileid;


// widgets tab 1: Distortion correction
var startStepDDBox, thisStepRadioButton, allStepsRadioButton;
var bandwidthBox, echotimediffBox, encdirBox, apply2piBox, chbCoregField2Func; 
var magnFileLabel, phaseFileLabel, unwrFileLabel, b0FileLabel, pixshiftFileLabel, epiFileLabel, vmrFileLabel, vmrFileButton;
var phaseMagnFileButton, unwrappedFileButton, b0FileButton, pixshiftFileButton, epiFileButton, proceedButton, startButton;
var fmrRadioButton, niiRadioButton, hdrRadioButton;
// widgets tab 2: Unwrapping options
var rbtn3DFMG, rbtnRelaxation, spbNrIterRelaxUnwrap, dspbRegularisation;
var spbNrVCycles, spbIterPerCycle, chbPhaseToRadians, chbMaskLaplacian;
// widgets tab 3: Evaluation
var btnGetDistImg, btnGetUndistImg, lblGetDistImg, lblGetUndistImg, btnCreateDiffImg;
var distfmr, undistfmr;
// widgets tab 4: Conversions
var input1Label, input2Label, input3Label, outputField, outputLabel, dspbInput1, dspbInput2, dspbInput3;
var rbtnDeg2Rad, rbtnRad2Deg, rbtnRad2Hz, rbtnPPM2Hz, rbtnES2PEBW, rbtnDFreq2Dx;
var btnConvertOtherCalc;
var btnConvertCart2Pol, btnConvertPol2Cart, leRealCoord, leImagCoord, leMagn, lePhase;
var fieldSIamount, boxSIamountTime, boxSIamountDistance, boxSIamountFrequency, btnConvertSIunits, secondsField, metersField, hertzField;
var btnGetRealFileName, btnGetImagFileName, btnConvertCartFiles2PolFiles, lblRealFileName, lblImagFileName; 
//var , btnConvertPPMFile2Hz, btnGetPPMFileName;*/


// check parameters
var startstep = FILE_STAGE_NOT_SET;
var unwrappingparamspresent = false;
var acqparamspresent = false;
var datatoselect = 0;
var procedure = "Unknown";
var docoreg = false;
var doAll = false;


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ functions tab 1: Distortion correction ~~~~~~~~~~~~~~~~~~~~

scriptObj.proceedToFileUpload = function() {

    var proceed = true;
       
    // read start step and acquisition parameters
    startstep = startStepDDBox.currentIndex;
    
    // get nr of steps
    doAll = allStepsRadioButton.checked; 
    
    switch (startstep) { 
        case FILE_STAGE_PHASE_UNWRAP:  
            procedure = "Unwrap";
            phaseMagnFileButton.enabled = true; 
            datatoselect = 2; // magnitude + phase
            if (doAll) {
                  epiFileButton.enabled = true; 
                  datatoselect += 1;
             if (chbCoregField2Func.checked) {
                    vmrFileButton.enabled = true; // 18-12-13: now try via GUI signals/slots         
                    datatoselect += 1;
                }              
            } 
            break; 
        case FILE_STAGE_PHASE_COMPUTE_B0: 
            procedure = "ComputeB0";
            unwrappedFileButton.enabled = true;
            datatoselect = 1;
            if (doAll) {
                  epiFileButton.enabled = true; 
                  datatoselect += 1;
                 if (chbCoregField2Func.checked) {
                    vmrFileButton.enabled = true; // 18-12-13: now try via GUI signals/slots         
                    datatoselect += 1;
                }                    
            }            
            break;
        case FILE_STAGE_PHASE_COMPUTE_PIXSH:
            procedure = "ComputePxshift";
            b0FileButton.enabled = true;
            datatoselect = 1;
            if (doAll) {
                epiFileButton.enabled = true; 
                datatoselect += 1;
                 if (chbCoregField2Func.checked) {
                    vmrFileButton.enabled = true; // 18-12-13: now try via GUI signals/slots         
                    datatoselect += 1;
                }                  
            }            
            break;
        case FILE_STAGE_EPI_UNDISTORT: 
            procedure = "Undistort";
            pixshiftFileButton.enabled = true;
            epiFileButton.enabled = true; 
            datatoselect = 2; // EPI + pixelshift
           if (chbCoregField2Func.checked) {
                vmrFileButton.enabled = true; // 18-12-13: now try via GUI signals/slots         
                datatoselect += 1;
            } 
            break;
        default:   
            proceed = false;
            BrainVoyagerQX.PrintToLog("No startstep selected. Please try again.");
    }

   // enable the radiobuttons
    if (proceed) {
            fmrRadioButton.enabled = true;
            niiRadioButton.enabled = false;
            hdrRadioButton.enabled = false;
    }

    if (proceed) {
        if (startstep == FILE_STAGE_PHASE_UNWRAP) var unwrparamsok = this.collectUnwrappingParameters();
        proceed = this.collectAcquisitionParameters();
    }

}

/*
scriptObj.getFiles = function(filestage){

    var success = true;
    var fileid = this.getFileType();
BrainVoyagerQX.PrintToLog("fileid: " + fileid);    
    return this.getFiles(filestage, fileid);
}*/
 
scriptObj.getFiles = function(filestage, fileid) {

    var success = true;
    if (fileid == undefined) {
//    var fileid = this.getFileType(); // >>> is niet goed, vraagt alleen FMR or NIfTI.
        fileid = this.getFileType();  
    }
  //  var fileid = this.getFileTypeFromExtension(); // 181213; lekker slim, je weet de filenaam nog niet!
    BrainVoyagerQX.SetPluginIntParameter("FileTypeId", fileid); // FMR, VMR or NIfTI; to add: DMR?
    if (fileid != BV_TYPE_VMR) {startfileid = fileid;
    }
    BrainVoyagerQX.SetPluginIntParameter("FileStage", filestage);
    BrainVoyagerQX.SetPluginStringParameter("Command", "GetFiles"); 
    BrainVoyagerQX.ExecutePlugin(); 
        // now as many files can be selected as possible
        // writes also a text file to disk with the names
    var filenamessize = BrainVoyagerQX.GetPluginIntParameter("NrOfFiles");
    BrainVoyagerQX.PrintToLog("Number of selected files: " + filenamessize);
    if (filenamessize == 0) {
            BrainVoyagerQX.PrintToLog("No files selected. Procedure canceled.");
            cancel = true;
            success = false;
    } else {
        datatoselect -= filenamessize;
    }
    return success;
}

scriptObj.getPhaseMagnFiles = function() {
	BrainVoyagerQX.PrintToLog("Get magnitude and/or wrapped phase map(s)..."); 
    var success = this.getFiles(FILE_STAGE_PHASE_UNWRAP);
    if (success) {
        var filename = BrainVoyagerQX.GetPluginStringParameter("WrPhaseFile");
        phaseFileLabel.text = filename;
        if (startfileid == BV_TYPE_FMR) BrainVoyagerQX.OpenDocument(filename);
        filename = BrainVoyagerQX.GetPluginStringParameter("MagnFile"); //// changed to this 06-12-10
        magnFileLabel.text = filename;
        if (startfileid == BV_TYPE_FMR) BrainVoyagerQX.OpenDocument(filename);
    }
}

scriptObj.getUnwrappedFiles = function() {
	BrainVoyagerQX.PrintToLog("Get unwrapped phase map(s)..."); 
    var success = this.getFiles(FILE_STAGE_PHASE_COMPUTE_B0);
    if (success) {
        var filename = BrainVoyagerQX.GetPluginStringParameter("UnwrPhaseFile");
        unwrFileLabel.text = filename;
        if (startfileid == BV_TYPE_FMR) BrainVoyagerQX.OpenDocument(filename); // new per 06-12-10
    }
}

scriptObj.getB0Files = function() {
	BrainVoyagerQX.PrintToLog("Get field inhomogeneity map(s)..."); 
    var success = this.getFiles(FILE_STAGE_PHASE_COMPUTE_PIXSH);
    if (success) {
        var filename = BrainVoyagerQX.GetPluginStringParameter("B0File");
        b0FileLabel.text = filename;
        if (startfileid == BV_TYPE_FMR) BrainVoyagerQX.OpenDocument(filename); // new per 06-12-10
    }
}

scriptObj.getPixelshiftFiles = function() {
	BrainVoyagerQX.PrintToLog("Get pixel shift map(s)..."); 
    var success = this.getFiles(FILE_STAGE_EPI_UNDISTORT);
    if (success) {
        var filename = BrainVoyagerQX.GetPluginStringParameter("PixshiftFile");
        pixshiftFileLabel.text = filename;
        if (startfileid == BV_TYPE_FMR) BrainVoyagerQX.OpenDocument(filename); // new per 06-12-10
    }
}

scriptObj.getEPIFiles = function() { 

	BrainVoyagerQX.PrintToLog("Get EPI data..."); 
    var success = this.getFiles(FILE_STAGE_EPI_REQUIRED);
    if (success) {
        var filename = BrainVoyagerQX.GetPluginStringParameter("EPIFile");
        epiFileLabel.text = filename;        
        if (startfileid == BV_TYPE_FMR) BrainVoyagerQX.OpenDocument(filename); // new per 06-12-10
    }
}

scriptObj.getVMRFile = function() { 

	BrainVoyagerQX.PrintToLog("Get VMR data..."); 
    var success = this.getFiles(FILE_STAGE_EPI_UNDISTORT_WITH_COREG, BV_TYPE_VMR);
    if (success) {
        var vmrname = BrainVoyagerQX.GetPluginStringParameter("VMRFile");
        vmrFileLabel.text = vmrname;
        BrainVoyagerQX.OpenDocument(vmrname); // new per 18-12-13
    }
}

scriptObj.getFileType = function() {

    var filetype;  
    if (fmrRadioButton.checked == 1) {
        filetype = BV_TYPE_FMR; 
    } else if (niiRadioButton.checked == 1) {
        filetype = NIFTI_FILE_TYPE;
    } else if (hdrRadioButton.checked == 1) {
        filetype = NIFTI_PAIR_FILE_TYPE;
    }     
    return filetype;
}


scriptObj.getFileExtension = function(filename) {

    var posOfDot = filename.lastIndexOf(".");    
    var extension = filename.substring((posOfDot+1), filename.length);
    return extension;
}


scriptObj.getFileTypeFromExtension = function(filename) {

    var filetype;    
    var ext = this.getFileExtension(filename);
        
    if (filename.indexOf("fmr") > -1) {
        filetype = BV_TYPE_FMR; 
    } else if (filename.indexOf("vmr") > -1) {
        filetype = BV_TYPE_VMR;     
    } else if (filename.indexOf("nii") > -1) {
        filetype = NIFTI_FILE_TYPE;
    } else if (filename.indexOf("hdr") > -1) {
        filetype = NIFTI_PAIR_FILE_TYPE;
    } 
    return filetype;
}


scriptObj.collectUnwrappingParameters = function() {

    var success = true;
    BrainVoyagerQX.PrintToLog("Unwrapping parameters:"); 
    
    if (rbtnRelaxation.checked == 1) {
    
        do3DFMG = false;
        nrofiterationsrelax = spbNrIterRelaxUnwrap.value;
        regularisation = dspbRegularisation.value;
        BrainVoyagerQX.PrintToLog("Number of iterations for relaxation: " + nrofiterationsrelax);
        BrainVoyagerQX.PrintToLog("Regularisation parameter: " + regularisation);
        
    } else {
    
        nrofcycles = spbNrVCycles.value;
        nrofiterations = spbIterPerCycle.value;
        phase2radians = chbPhaseToRadians.checked; 
        masklaplacian = chbMaskLaplacian.checked;
        BrainVoyagerQX.PrintToLog("Number of V-Cycles for unwrapping: " + nrofcycles); 
        BrainVoyagerQX.PrintToLog("Number of iterations: " + nrofiterations); 
        BrainVoyagerQX.PrintToLog("Convert phase data to radians (-pi to pi) before unwrapping: " + (phase2radians ? "yes" : "no")); 
        BrainVoyagerQX.PrintToLog("Use mask for Laplacian to reduce influence of noise: " + (masklaplacian ? "yes" : "no"));  
    }  
    
    // check parameters
    if (do3DFMG) {
        if (nrofcycles == 0 || nrofiterations == 0) {
            BrainVoyagerQX.PrintToLog("One or more of the unwrapping parameters is 0. Unwrapping results may be suboptimal.");
            success = false;
        } else {
            unwrappingparamspresent = true;
        }
    } else {
        if (nrofiterationsrelax == 0) {
            BrainVoyagerQX.PrintToLog("Number of iterations for unwrapping is 0. Unwrapping results may be suboptimal.");
            success = false;
        } else {
            unwrappingparamspresent = true;
        }
    }
    
    if (!phase2radians){
        BrainVoyagerQX.PrintToLog("Please note that the conversion to radians, which is necessary in most cases, is switched off.")
    } 
    return success;
}

scriptObj.collectAcquisitionParameters = function() { 

      var success = true;
      BrainVoyagerQX.PrintToLog("Acquisition parameters:"); 
      bandwidthpe = bandwidthBox.value;
      echotimediff = echotimediffBox.value;
      phasencodingdir = encdirBox.currentIndex;
      // here is a circumvention because other directions have been removed (25-10-11)      
      if (phasencodingdir == 1) { // +y
        phasencodingdir = 3;
      } else if (phasencodingdir == 2) { // -y
        phasencodingdir = 4;
      }
      
      apply2pi = false;
      BrainVoyagerQX.PrintToLog("Bandwidth in phase encoding direction: " + bandwidthpe);
      BrainVoyagerQX.PrintToLog("Echo time difference (TE2 - TE1): " + echotimediff);
      BrainVoyagerQX.PrintToLog("Phase encoding direction, text: " + encdirBox.currentText + " (" + phasencodingdir + ")"); 
      
      // check parameters
      if (bandwidthpe == 0 || echotimediff == 0.0 || phasencodingdir == 0) {
        BrainVoyagerQX.PrintToLog("One or more of the acquisition parameters is 0");
        success = false;
      } else {
        acqparamspresent = true;
      }
      docoreg = chbCoregField2Func.checked; // 11-12-13
      return success;
}


scriptObj.startProcessing = function() {

    BrainVoyagerQX.PrintToLog("Start processing..."); 
        
    if ((procedure != "Unknown") && (startstep > 0) && (datatoselect == 0)) { 
        
        if (((startstep == FILE_STAGE_PHASE_UNWRAP) && (!doAll) || acqparamspresent)) {
        
            // set plugin parameters
            BrainVoyagerQX.SetPluginStringParameter("Command", "Start");
            BrainVoyagerQX.SetPluginStringParameter("Procedure", procedure);
            
            BrainVoyagerQX.SetPluginIntParameter("UnwrappingMethod", (do3DFMG ? 1 : 0));
            if (do3DFMG) {
                BrainVoyagerQX.SetPluginIntParameter("NrOfVCycles", nrofcycles); // for unwrapping, 21-02-11
                BrainVoyagerQX.SetPluginIntParameter("NrOfIterations", nrofiterations); // for unwrapping, 21-02-11
                BrainVoyagerQX.SetPluginIntParameter("PhaseToRadians", (phase2radians ? 1 : 0)); // 07-06-11
                BrainVoyagerQX.SetPluginIntParameter("MaskLaplacian", (masklaplacian ? 1 : 0)); // 07-06-11
            } else {
                BrainVoyagerQX.SetPluginIntParameter("NrOfIterationsRelax", nrofiterationsrelax); // 11-10-11
                BrainVoyagerQX.SetPluginFloatParameter("Regularisation", regularisation);
            }
            
            BrainVoyagerQX.SetPluginFloatParameter("BandWidthPE", bandwidthpe);
            BrainVoyagerQX.SetPluginFloatParameter("EchoTimeDiff", echotimediff);
            BrainVoyagerQX.SetPluginIntParameter("PhaseEncodingDir", phasencodingdir);    
            BrainVoyagerQX.SetPluginIntParameter("ApplyTwoPi", (apply2pi ? 1 : 0)); // 19-07-11  
            BrainVoyagerQX.SetPluginIntParameter("PerformAllSteps", (doAll ? 1 : 0));   
            BrainVoyagerQX.SetPluginIntParameter("StartFileTypeId", startfileid); // FMR, (DMR) or NIfTI
            
            // get filenames from textlabels (this is redundant, will be changed in later version)     
            var startfile, epifile;
            switch (startstep) {
                case FILE_STAGE_PHASE_UNWRAP:               startfile = phaseFileLabel.text;    break;
                case FILE_STAGE_PHASE_COMPUTE_B0:           startfile = unwrFileLabel.text;     break;
                case FILE_STAGE_PHASE_COMPUTE_PIXSH:        startfile = b0FileLabel.text;       break;
                case FILE_STAGE_EPI_UNDISTORT:              startfile = pixshiftFileLabel.text; break;
            }       
            BrainVoyagerQX.SetPluginStringParameter("StartFile", startfile);        
            if (startstep == FILE_STAGE_PHASE_UNWRAP) {
                BrainVoyagerQX.SetPluginStringParameter("MagnFile", magnFileLabel.text);
            }
            if ((doAll) || (startstep == FILE_STAGE_EPI_UNDISTORT)) {
                BrainVoyagerQX.SetPluginIntParameter("mapFieldmapToFunctional", chbCoregField2Func.checked);
                if (chbCoregField2Func.checked) {
                    BrainVoyagerQX.SetPluginStringParameter("VMRFile", vmrFileLabel.text);
                }
                epifile = epiFileLabel.text;     
                BrainVoyagerQX.SetPluginStringParameter("EPIFile", epifile);       
                BrainVoyagerQX.SetPluginIntParameter("EPIFileTypeId", startfileid); // FMR or NIfTI            
            }
                                       
            // now start processing      
            this.anatabacusDlg.close();                            
            BrainVoyagerQX.ExecutePlugin();
            
            // ask parameters from plugin
            if ((this.checkerrorstatus() > -1) && ((doAll) || (startstep == FILE_STAGE_EPI_UNDISTORT))) {
            
                var distortedFMR = BrainVoyagerQX.ActiveDocument; // because the EPI is the last selected image, so is the active document
                distortedFMR.Close();        
           //     var undistortedname = BrainVoyagerQX.GetPluginStringParameter("NameUndistortedFile"); // this is set in undistort1d() in plugin (without path)
                var undistortedname = extendFileName(epifile, "undistort");
                BrainVoyagerQX.PrintToLog("Opening " + undistortedname + "...");
                var undistortedFMR = BrainVoyagerQX.OpenDocument(undistortedname.toString());
            }
            
        } else {
            BrainVoyagerQX.PrintToLog("Missing acquisition parameters.");
        }
    
    } else {
        BrainVoyagerQX.PrintToLog("Procedure not selected or missing filenames.");
  //      BrainVoyagerQX.PrintToLog("procedure != \"Unknown\"): " + procedure + "(startstep > 0): " + startstep + "(datatoselect == 0): " + datatoselect);
    }
     
    // get feedback
//    var errornr = this.checkerrorstatus(); // prints error message to log tab, if there is one    
   // BrainVoyagerQX.PrintToLog("~~ closing anatabacus ~~");
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ functions tab 2: Evaluation ~~~~~~~~~~~~~~~~~~~~

scriptObj.getDistortedImageFile = function() {
	BrainVoyagerQX.PrintToLog("Get distorted image file..."); 
    var filename = this.getFile(BV_TYPE_FMR);
    if (filename.length > 0) {
        lblGetDistImg.text = filename;
        distfmr = BrainVoyagerQX.OpenDocument(filename); 
    }
}

scriptObj.getUndistortedImageFile = function() {
	BrainVoyagerQX.PrintToLog("Get undistorted image file..."); 
    var filename = this.getFile(BV_TYPE_FMR);
    if (filename.length > 0) {
        lblGetUndistImg.text = filename;
        undistfmr = BrainVoyagerQX.OpenDocument(filename);
    }
}

scriptObj.createDifferenceImage = function() {

    BrainVoyagerQX.PrintToLog("difference image... ");
    var distortedFile = lblGetDistImg.text; // these images have been opened in BrainVoyager after file selection
    var undistortedFile = lblGetUndistImg.text;
    if ((distortedFile.length > 1) && (undistortedFile.length > 1)) {
    
        // create difference image
        BrainVoyagerQX.PrintToLog("Create difference image of \n" + distortedFile + " and \n" + undistortedFile);
        BrainVoyagerQX.SetPluginStringParameter("Command", "Evaluate"); 
        BrainVoyagerQX.ExecutePlugin();    
        this.anatabacusDlg.close();
        distfmr.Close();
        undistfmr.Close();        
        
        var errornr = this.checkerrorstatus();
        if (errornr > -1) { 
            // show results
            var diffimgfmr = BrainVoyagerQX.GetPluginStringParameter("DifferenceImgName");
            BrainVoyagerQX.OpenDocument(diffimgfmr); 
            distfmr = BrainVoyagerQX.OpenDocument(distortedFile);
            undistfmr = BrainVoyagerQX.OpenDocument(undistortedFile);        
        }
    } else {
        BrainVoyagerQX.PrintToLog("Invalid file names for creating difference image.");
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ functions tab 3: Conversions ~~~~~~~~~~~~~~~~~~~~


scriptObj.getRealFileName = function() {
	BrainVoyagerQX.PrintToLog("Get real file..."); 
    var success = this.getFile(BV_TYPE_FMR);
    if (success) {
        var filename = BrainVoyagerQX.GetPluginStringParameter("FileName");
        lblRealFileName.text = filename;
        BrainVoyagerQX.OpenDocument(filename); 
    }
}

scriptObj.getImaginaryFileName = function() {
	BrainVoyagerQX.PrintToLog("Get imaginary file..."); 
    var success = this.getFile(BV_TYPE_FMR);
    if (success) {
        var filename = BrainVoyagerQX.GetPluginStringParameter("FileName");
        lblImagFileName.text = filename;
        BrainVoyagerQX.OpenDocument(filename); 
    }
}

scriptObj.getPPMFileName = function() {
	BrainVoyagerQX.PrintToLog("Get parts-per-million file..."); 
    var success = this.getFile(BV_TYPE_FMR);
    if (success) {
        var filename = BrainVoyagerQX.GetPluginStringParameter("FileName");
        lblPPMfileName.text = filename;
        if (startfileid == BV_TYPE_FMR) BrainVoyagerQX.OpenDocument(filename); 
    }
}


scriptObj.convertSIunits = function() {

    var siAmount = fieldSIamount.text;
    var selectedSIquantity;
    
    if (!isNaN(siAmount)) {
    
    // time
    if (boxSIamountTime.currentIndex > 0) {
        selectedSIquantity = boxSIamountTime.currentText;
        var quantityInSec;
        if (selectedSIquantity == "minutes") {
            quantityInSec = (siAmount * 60);
        } else if (selectedSIquantity == "milliseconds") {
            quantityInSec = (siAmount * Math.pow(10,-3));
        } else if (selectedSIquantity == "microseconds") {
            quantityInSec = (siAmount * Math.pow(10,-6));
        } else if (selectedSIquantity == "nanoseconds") {
            quantityInSec = (siAmount * Math.pow(10,-9));
        }
        secondsField.text = quantityInSec;
    }
    
    // position
    if (boxSIamountDistance.currentIndex > 0) {
        selectedSIquantity = boxSIamountDistance.currentText;
        var quantityInMeters;
        if (selectedSIquantity == "millimeter") {
            quantityInMeters = (siAmount * Math.pow(10,-3));
        } else if (selectedSIquantity == "centimeter") {
            quantityInMeters = (siAmount / 100);
        }    
        metersField.text = quantityInMeters;  
    }
    
    if (boxSIamountFrequency.currentIndex > 0) {
        selectedSIquantity = boxSIamountFrequency.currentText;
        var quantityInHz;
        if (selectedSIquantity == "kilohertz") {
            quantityInHz = (siAmount * Math.pow(10,3));
        } else if (selectedSIquantity == "megahertz") {
            quantityInHz = (siAmount * Math.pow(10,6));
        }  else if (selectedSIquantity == "gigahertz") {
            quantityInHz = (siAmount * Math.pow(10,9));
        }         
        hertzField.text = quantityInHz;
    }

    } else {
        BrainVoyagerQX.PrintToLog("Convert to SI units: input value(s) not-a-number (NaN).");
    }
}

scriptObj.convertCartesianToPolar = function() {

    BrainVoyagerQX.PrintToLog("Convert cartesian to polar coordinates...");
    var isPolar = false;    
    var realval = leRealCoord.text;
    var imagval = leImagCoord.text;
    var z = new Complex(realval, imagval, isPolar); 
    if (!isNaN(realval) && !isNaN(imagval)) { 
        leMagn.text = z.magnitude().toFixed(4);
        lePhase.text = z.phase().toFixed(4); 
    } else {
        BrainVoyagerQX.PrintToLog("Convert cartesian to polar: input value(s) not-a-number (NaN).");  
    }
}

scriptObj.convertPolarToCartesian = function() { 

    var isPolar = true;
    var magnitude = leMagn.text;
    var phase = lePhase.text; 
    var z = new Complex(magnitude, phase, isPolar);
    if (!isNaN(magnitude) && !isNaN(phase)) { 
            leRealCoord.text = z.real.toFixed(4); 
            leImagCoord.text = z.imag.toFixed(4); 
    } else {
        BrainVoyagerQX.PrintToLog("Convert polar to cartesian: input value(s) not-a-number (NaN).");  
    }
}

scriptObj.convertCartesianToPolarFiles = function() {

    var realfile = lblRealFileName.text;
    var imagfile = lblImagFileName.text;
    
  //     var success = true;
    BrainVoyagerQX.SetPluginStringParameter("Command", "CreatePolarFile"); 
    BrainVoyagerQX.SetPluginStringParameter("RealFile", realfile);
    BrainVoyagerQX.SetPluginStringParameter("ImagFile", imagfile);
    
    this.anatabacusDlg.close();  
    BrainVoyagerQX.ExecutePlugin(); 
  //  var filename = BrainVoyagerQX.GetPluginStringParameter("FileName");
//BrainVoyagerQX.PrintToLog("Received filename: " + filename);    
    
//    return filename;
  
    
}

scriptObj.convertPPMFile2Hz = function() {

}
   
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ functions tab 3: conversions: other calculations ~~~~~~~~~~~~~~~~~~~~ 

scriptObj.clearOtherCalcFields = function() {

    dspbInput1.value = 0.0; 
    dspbInput2.value = 0.0; 
    dspbInput3.value = 0.0;
    outputField.text = "";
}

scriptObj.setDeg2RadLabel = function() {

    input1Label.text = "degrees:";// (0-360)
    input2Label.text = "";
    input3Label.text = "";    
    outputLabel.text = "radians:";// (in pi)
    this.clearOtherCalcFields();
}

scriptObj.setRad2DegLabel = function() {

    input1Label.text = "radians:"; // (in pi)
    input2Label.text = "";
    input3Label.text = ""; 
    outputLabel.text = "degrees:";
    this.clearOtherCalcFields();
}

scriptObj.setRad2HzLabel = function() {

    input1Label.text = "radians:";// (in pi)
    input2Label.text = "";
    input3Label.text = "";     
    outputLabel.text = "hertz:";
    this.clearOtherCalcFields();
}

scriptObj.setPPM2HzLabels = function() {

    input1Label.text = "parts per million (ppm):";
    input2Label.text = "field strength (T):";
    input3Label.text = "";//Larmor frequency (Hz)     
    outputLabel.text = "hertz:";
    this.clearOtherCalcFields();
}

scriptObj.setES2PEBWLabels = function() {

    input1Label.text = "echospacing (msec):";
    input2Label.text = "matrix size y:";
 //   input3Label.text = "Fourier encoding (%):";
    input3Label.text = "acceleration factor (phase enc dir):";
    outputLabel.text = "bandwidth (phase enc dir)(Hz):";
    this.clearOtherCalcFields();
}

scriptObj.setDFreq2DxLabels = function() {

    input1Label.text = "bandwidth (phase enc dir)(Hz):";
    input2Label.text = "B0 field inhomogeneity (Hz):";
    input3Label.text = "";
    outputLabel.text = "pixel shift:";
    this.clearOtherCalcFields();
}

scriptObj.convertOtherEntities = function() {


    var param1 = dspbInput1.value; 
    var param2 = dspbInput2.value; 
    var param3 = dspbInput3.value;
    var result = 0.0;
 
    if (rbtnDeg2Rad.checked) {
        // convert degrees to radians
        BrainVoyagerQX.PrintToLog("Convert degrees to radians...");
        result = param1 * (Math.PI / 180);
        
    } else if (rbtnRad2Deg.checked) {
        // convert radians to degrees
        BrainVoyagerQX.PrintToLog("Convert radians to degrees...");
        result = param1 * (180 / Math.PI);
    
    } else if (rbtnRad2Hz.checked) {
        // convert radians to hertz
        BrainVoyagerQX.PrintToLog("Convert radians to Hertz...");
        result = param1 / (2 * Math.PI);
            
    } else if (rbtnPPM2Hz.checked) {
        // convert parts-per-million to hertz
        BrainVoyagerQX.PrintToLog("Convert parts-per-million (ppm) to Hertz (Hz) for 1H...");
        var ppm = param1 * (1/1000000);
        var resonancefreq = 42576000 * param2;
        result = ppm * resonancefreq; 
    
    } else if (rbtnES2PEBW.checked) {
        // convert echospacing to phase encoding bandwidth
        BrainVoyagerQX.PrintToLog("Convert echospacing (esp) to phase encoding bandwidth...");
        var echospacing_sec = param1 * (1/1000); 
        BrainVoyagerQX.PrintToLog("Echospacing in seconds: " + echospacing_sec);
        var ny = param2; 
    /*    var fourierperc = (param3 / 100);  
        BrainVoyagerQX.PrintToLog("Number of phase encoding lines: " + ny + ", Fourier partial scanning: " + fourierperc);
        result = 1 / (echospacing_sec * (ny * fourierperc));  
        BrainVoyagerQX.PrintToLog("bandwidth in phase encoding direction = 1 / (echospacing_sec * (ny * Fourier partial scanning)) = " + result.toPrecision(10));
    */  var accelfac = param3;  
        if ((accelfac == "") || (accelfac == 0)) accelfac = 1;
        var echospacing_sec_acc = echospacing_sec / accelfac; 
        BrainVoyagerQX.PrintToLog("Number of phase encoding lines: " + ny + ", SENSE/GRAPPA acceleration factor in phase encoding direction: " + accelfac);
        result = 1 / (echospacing_sec_acc * ny);  
        BrainVoyagerQX.PrintToLog("bandwidth in phase encoding direction = 1 / (echospacing_sec_acc * ny) = " + result.toPrecision(10));  
    } else if (rbtnDFreq2Dx.checked) {
        // convert frequency shift to spatial shift in pixels
        BrainVoyagerQX.PrintToLog("Convert frequency shift to spatial shift in pixels...");
        
        bandwidth_pe_hz = param1;
        deltab0 = param2;
        result = (deltab0 / bandwidth_pe_hz); 
    }
    outputField.text = result.toPrecision(10);

}


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ generic functions ~~~~~~~~~~~~~~~~~~~~ 

scriptObj.getFile = function(fileid) {

    var success = true;
    BrainVoyagerQX.SetPluginIntParameter("FileTypeId", fileid); // FMR or NIfTI
    BrainVoyagerQX.SetPluginStringParameter("Command", "GetFile"); 
    BrainVoyagerQX.ExecutePlugin(); 
    var filename = BrainVoyagerQX.GetPluginStringParameter("FileName");
    return filename;
}



scriptObj.checkerrorstatus = function() {
    var errornr = BrainVoyagerQX.GetPluginIntParameter("ErrorNr");
    if (errornr < 0) {
            var errormsg = BrainVoyagerQX.GetPluginStringParameter("ErrorMsg");
            BrainVoyagerQX.PrintToLog(errormsg);
            cancel = true;
    } 
    return errornr;
}

scriptObj.initDlg = function() { 

    this.anatabacusDlg.windowTitle = "anatabacus"; 
    
    // procedure widgets
    startStepDDBox  = BrainVoyagerQX.FindChild("cmboxStartStep");
    thisStepRadioButton = BrainVoyagerQX.FindChild("rbtnThisStepOnly"); 
    allStepsRadioButton = BrainVoyagerQX.FindChild("rbtnAllNextSteps");
    
    // unwrapping parameter widgets
    rbtn3DFMG           = BrainVoyagerQX.FindChild("rbtn3DFMG");
    rbtnRelaxation      = BrainVoyagerQX.FindChild("rbtnRelaxation");
    spbNrIterRelaxUnwrap    = BrainVoyagerQX.FindChild("spbNrIterRelaxUnwrap");
    dspbRegularisation  = BrainVoyagerQX.FindChild("dspbRegularisation"); 
    spbNrVCycles        = BrainVoyagerQX.FindChild("spbNrVCycles");
    spbIterPerCycle     = BrainVoyagerQX.FindChild("spbIterPerCycle");  
    chbPhaseToRadians   = BrainVoyagerQX.FindChild("chbPhaseToRadians");
    chbMaskLaplacian    = BrainVoyagerQX.FindChild("chbMaskLaplacian"); 
    
     // acquisition parameter widgets
    bandwidthBox        = BrainVoyagerQX.FindChild("spnBoxBW");
    echotimediffBox     = BrainVoyagerQX.FindChild("dSpnBoxDTE");
    encdirBox           = BrainVoyagerQX.FindChild("cmBoxPEDir");
    
    // coreg widget
    chbCoregField2Func  = BrainVoyagerQX.FindChild("chbCoregField2Func");

    // file loading widgets
    fmrRadioButton      = BrainVoyagerQX.FindChild("rbtnFMR");
    niiRadioButton      = BrainVoyagerQX.FindChild("rbtnNII");
    hdrRadioButton      = BrainVoyagerQX.FindChild("rbtnHDR");
    magnFileLabel       = BrainVoyagerQX.FindChild("lblMagnFile");
    phaseFileLabel      = BrainVoyagerQX.FindChild("lblPhaseFile");
    unwrFileLabel       = BrainVoyagerQX.FindChild("lblUnwrFile");
    b0FileLabel         = BrainVoyagerQX.FindChild("lblB0File");
    pixshiftFileLabel   = BrainVoyagerQX.FindChild("lblPixshiftFile");
    epiFileLabel        = BrainVoyagerQX.FindChild("lblEPIfile");
    phaseMagnFileButton = BrainVoyagerQX.FindChild("btnGetPhaseMagn");
    unwrappedFileButton = BrainVoyagerQX.FindChild("btnGetUnwrapped"); 
    b0FileButton        = BrainVoyagerQX.FindChild("btnGetB0");  
    pixshiftFileButton  = BrainVoyagerQX.FindChild("btnGetPixshift"); 
    epiFileButton       = BrainVoyagerQX.FindChild("btnGetEPI"); 
    vmrFileButton       = BrainVoyagerQX.FindChild("btnGetVMR"); 
    vmrFileLabel        = BrainVoyagerQX.FindChild("lblVMRfile");
    btnGetDistImg       = BrainVoyagerQX.FindChild("btnGetDistImg"); 
    btnGetUndistImg     = BrainVoyagerQX.FindChild("btnGetUndistImg"); 
    btnGetRealFileName  = BrainVoyagerQX.FindChild("btnGetRealFileName");
    btnGetImagFileName  = BrainVoyagerQX.FindChild("btnGetImagFileName");
    btnConvertCartFiles2PolFiles = BrainVoyagerQX.FindChild("btnConvertCartFiles2PolFiles");
    lblRealFileName     = BrainVoyagerQX.FindChild("lblRealFileName"); 
    lblImagFileName     = BrainVoyagerQX.FindChild("lblImagFileName"); 
    
    // action widgets
    proceedButton       = BrainVoyagerQX.FindChild("btnProceed");
    startButton         = BrainVoyagerQX.FindChild("btnStart");
    
    // evaluation widgets
    lblGetDistImg = BrainVoyagerQX.FindChild("lblGetDistImg");
    lblGetUndistImg = BrainVoyagerQX.FindChild("lblGetUndistImg");
    btnCreateDiffImg = BrainVoyagerQX.FindChild("btnCreateDiffImg");
    
    // conversion widgets
    input1Label = BrainVoyagerQX.FindChild("input1Label");
    input2Label = BrainVoyagerQX.FindChild("input2Label"); 
    input3Label = BrainVoyagerQX.FindChild("input3Label"); 
    dspbInput1 = BrainVoyagerQX.FindChild("dspbInput1");
    dspbInput2 = BrainVoyagerQX.FindChild("dspbInput2"); 
    dspbInput3 = BrainVoyagerQX.FindChild("dspbInput3");  
    outputLabel = BrainVoyagerQX.FindChild("outputLabel");    
    outputField = BrainVoyagerQX.FindChild("outputField"); 
    btnConvertOtherCalc = BrainVoyagerQX.FindChild("btnConvertOtherCalc");     
    btnConvertCart2Pol = BrainVoyagerQX.FindChild("btnConvertCart2Pol");
    btnConvertPol2Cart = BrainVoyagerQX.FindChild("btnConvertPol2Cart");
    leRealCoord = BrainVoyagerQX.FindChild("leRealCoord");
    leImagCoord = BrainVoyagerQX.FindChild("leImagCoord");
    leMagn = BrainVoyagerQX.FindChild("leMagn");
    lePhase = BrainVoyagerQX.FindChild("lePhase");
    fieldSIamount = BrainVoyagerQX.FindChild("leSIinput");
    boxSIamountTime = BrainVoyagerQX.FindChild("cmbTime");
    boxSIamountDistance = BrainVoyagerQX.FindChild("cmbDistance");
    boxSIamountFrequency = BrainVoyagerQX.FindChild("cmbFrequency");
    btnConvertSIunits = BrainVoyagerQX.FindChild("btnConvertSIunits");
    secondsField = BrainVoyagerQX.FindChild("leSeconds"); 
    metersField = BrainVoyagerQX.FindChild("leMeters");
    hertzField = BrainVoyagerQX.FindChild("leHertz");
  /* btnConvertPPMFile2Hz = BrainVoyagerQX.FindChild("btnConvertPPMFile2Hz");
   btnGetPPMFileName = BrainVoyagerQX.FindChild("btnGetPPMFileName");
  */
   rbtnDeg2Rad = BrainVoyagerQX.FindChild("rbtnDeg2Rad");
   rbtnRad2Deg = BrainVoyagerQX.FindChild("rbtnRad2Deg");
   rbtnRad2Hz = BrainVoyagerQX.FindChild("rbtnRad2Hz");
   rbtnPPM2Hz = BrainVoyagerQX.FindChild("rbtnPPM2Hz");
   rbtnES2PEBW = BrainVoyagerQX.FindChild("rbtnES2PEBW");
   rbtnDFreq2Dx = BrainVoyagerQX.FindChild("rbtnDFreq2Dx");

    // connect proceed to file uploading button with function
    proceedButton.clicked.connect(this, this.proceedToFileUpload);
    
    // connect file buttons to file dialog functions
    phaseMagnFileButton.clicked.connect(this, this.getPhaseMagnFiles);
    unwrappedFileButton.clicked.connect(this, this.getUnwrappedFiles); 
    b0FileButton.clicked.connect(this, this.getB0Files);  
    pixshiftFileButton.clicked.connect(this, this.getPixelshiftFiles); 
    epiFileButton.clicked.connect(this, this.getEPIFiles); 
    vmrFileButton.clicked.connect(this, this.getVMRFile);
    btnGetDistImg.clicked.connect(this, this.getDistortedImageFile);
    btnGetUndistImg.clicked.connect(this, this.getUndistortedImageFile);
    btnCreateDiffImg.clicked.connect(this, this.createDifferenceImage);
        
    // conversions: other calculations    
    rbtnDeg2Rad.toggled.connect(this, this.setDeg2RadLabel);    
    rbtnRad2Deg.toggled.connect(this, this.setRad2DegLabel);
    rbtnRad2Hz.toggled.connect(this, this.setRad2HzLabel);
    rbtnPPM2Hz.toggled.connect(this, this.setPPM2HzLabels);
    rbtnES2PEBW.toggled.connect(this, this.setES2PEBWLabels);
    rbtnDFreq2Dx.toggled.connect(this, this.setDFreq2DxLabels);
    btnConvertOtherCalc.clicked.connect(this, this.convertOtherEntities); 
        
    btnGetRealFileName.clicked.connect(this, this.getRealFileName);
    btnGetImagFileName.clicked.connect(this, this.getImaginaryFileName);
   // btnGetPPMFileName.clicked.connect(this, this.getPPMFileName);// 2. evaluate button

    
    // 3. conversion buttons
    btnConvertCart2Pol.clicked.connect(this, this.convertCartesianToPolar);
   btnConvertPol2Cart.clicked.connect(this, this.convertPolarToCartesian); 
    btnConvertSIunits.clicked.connect(this, this.convertSIunits);
   btnConvertCartFiles2PolFiles.clicked.connect(this, this.convertCartesianToPolarFiles);
 /*  btnConvertPPMFile2Hz.clicked.connect(this, this.convertPPMFile2Hz);
   */      
    // connect 'Start processing' button to start processing function
    startButton.clicked.connect(this, this.startProcessing);
    
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
/* constructor of complex number object */
function Complex(x, y, isPolar) {

    BrainVoyagerQX.PrintToLog("Create complex number...");
    if (isPolar) {
        	this.real =	x * Math.cos(y);	
			this.imag = x * Math.sin(y);	
    } else {
        this.real = x;
        this.imag = y;
    }    
    BrainVoyagerQX.PrintToLog(this.real + " + " + this.imag + "i");
    
    // declaration of member functions
    this.magnitude = magnitude;
    this.phase = phase;
}

function magnitude() {
    var magnitude = Math.sqrt(Math.pow(this.real,2) + Math.pow(this.imag,2));
    return magnitude; 
}

function phase() {
    var phase = Math.atan2(this.imag, this.real);
    return phase; 
}
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

// This function adds a substring at the end of the name (before the extension): <old name>_<new string>.<old extension>
function extendFileName(filename, substr) {

	var index = filename.lastIndexOf(".");
	if (index != -1) {
		var basename = filename.substring(0, index);
		var extension = filename.substring(index, filename.length);
		var newname = basename + "_" + substr + extension;
	}
	return newname;
}

returnScriptObj = function() { 
	return scriptObj; 
} 

returnScriptObj(); // should be after returnScriptOjb = function()
